/*
 * mtsParameters.h
 *
 *  Created on: Feb 6, 2013
 *      Author: sbak
 */

#ifndef MTSPARAMETERS_H_
#define MTSPARAMETERS_H_

#include "common.h"
#include "CovarianceMatrix.h"
#include "SignatureModel.h"

#include <sys/time.h>

#include <QFile>

//MODEL parameters
int mts_equalization;
float mts_resolution;
float mts_mask_density;
int mts_use_integrals;
std::string mts_model;
std::string mts_weight_kernel;

time_t getTimeStamp(){

	return time(NULL);
}

QDomElement saveParameters(){

	QDomDocument doc("NULL");
	QDomElement param = doc.createElement("PARAMETERS");
	param.setAttribute("MODEL", mts_model.c_str());
	param.setAttribute("USE_INTEGRALS", mts_use_integrals);
	param.setAttribute("EQUALIZATION", mts_equalization);
	param.setAttribute("RESOLUTION", mts_resolution);
	param.setAttribute("MASK_DENSITY", mts_mask_density);
	param.setAttribute("WEIGHT_KERNEL", mts_weight_kernel.c_str());

	return param;
}

void mts_saveParameters(std::string filename) {

	QDomDocument doc("MTS_CONFIG");
	QDomElement root = saveParameters();
	doc.appendChild(root);

	QFile file((filename).c_str());
	if (!file.open(QIODevice::WriteOnly)) {
		return;
	}

	QTextStream ts(&file);
	ts << doc.toString();
	file.close();
}

void loadParameters(QDomElement root){

	mts_model =  root.attribute("MODEL","NULL").toStdString();
	mts_use_integrals = root.attribute("USE_INTEGRALS", "1").toInt();
	mts_equalization = root.attribute("EQUALIZATION", "1").toInt();
	mts_resolution = root.attribute("RESOLUTION", "0.2").toFloat();
	mts_mask_density = root.attribute("MASK_DENSITY", "0.8").toFloat();
	mts_weight_kernel = root.attribute("WEIGHT_KERNEL", "Uniform").toStdString();
}

bool mts_loadParameters(std::string filename){

	QFile file((filename).c_str());
	if (!file.open(QIODevice::ReadOnly)) {

		_ERROR("file.open( " + filename + " ) crashed!");
		return false;
	}
	QDomDocument doc;
	if (!doc.setContent(&file)) {

		_ERROR("doc.setContent( " + filename + " ) crashed!");
		file.close();
		return false;
	}
	QDomElement root = doc.documentElement();
	if (root.tagName() != "PARAMETERS") {
		_ERROR("BAD XML file!");
		file.close();
		return false;
	}

	loadParameters(root);

	file.close();

	return true;
}

void img2img(IplImage** input, lti::channel* mask){

	if(mask == NULL){
		return;
	}else{

		assert((*input)->width == mask->columns());
		assert((*input)->height == mask->rows());

		IplImage* src = *input;
		for (int i = 0; i < src->height; i++) {
				for (int j = 0; j < src->width; j++) {

					int px = j * 3;
					int py = i;

					if (mask->at(i, j) == 0) {
						(src->imageData + src->widthStep * py)[px + 2] = 0;
						(src->imageData + src->widthStep * py)[px + 1] = 0;
						(src->imageData + src->widthStep * py)[px] = 0;
					}
				}
			};
	}
}
IplImage* preprocessIMG(IplImage* input, SignatureModel* model, lti::channel* mask = NULL){

	CvSize human_size = cvSize(model->width, model->height);
	IplImage* output = cvCreateImage(human_size, input->depth, input->nChannels);

	img2img(&input, mask);
	IplImage* cropped = getRectImage(input, mts_resolution);


	if(mts_equalization){
		if(mask == NULL){
			cvEqualize3Hist(cropped, cropped);
		}else{
			cvEqualize3HistMask(cropped, cropped);
		}
	}

	cvResize(cropped, output);
	cvReleaseImage(&cropped);

	return output;
}


#endif /* MTSPARAMETERS_H_ */
