/*
 * CSAllTypes.h
 *
 *  Created on: 14 avr. 2010
 *      Author: bboulay
 */



#ifndef CSBOOL_H_
#define CSBOOL_H_

#include <CSBasicType.h>

class CSBool : public CSBasicType {
public:
	CSBool();
	CSBool(const CSBool& i_value);
	CSBool(const CSBasicType& i_value);
	CSBool(const bool& i_value);
	CSBool(const bool& i_value, double i_confidence);
	virtual ~CSBool();

	bool operator==(const CSBool& i_object);
	bool operator!=(const CSBool& i_object);

	friend std::ostream& operator<<(std::ostream& i_os, const CSBool& i_object);


public:
	bool m_value;
};

#endif /* CSSTRING_H_ */



#ifndef CSSTRING_H_
#define CSSTRING_H_

#include <CSBasicType.h>

class CSString : public CSBasicType {
public:
	CSString();
	CSString(std::string i_value);
	CSString(const CSString& i_value);
	CSString(const CSBasicType& i_value);
	virtual ~CSString();

	bool operator==(const CSString& i_object);
	bool operator!=(const CSString& i_object);

	friend std::ostream& operator<<(std::ostream& i_os, const CSString& i_object);

public:
	std::string m_value;
};

#endif /* CSSTRING_H_ */

#ifndef CSINT_H_
#define CSINT_H_

#include <CSBasicType.h>

class CSInt : public CSBasicType {
public:
	CSInt();
	CSInt(int i_value);
	CSInt(const CSInt& i_value);
	CSInt(const CSBasicType& i_value);
	virtual ~CSInt();

	bool operator==(const CSInt& i_object);
	bool operator!=(const CSInt& i_object);

	friend std::ostream& operator<<(std::ostream& i_os, const CSInt& i_object);

public:
	int m_value;
};

#endif /* CSINT_H_ */

#ifndef CSDOUBLE_H_
#define CSDOUBLE_H_

#include <CSBasicType.h>

class CSDouble : public CSBasicType {
public:
	CSDouble();
	CSDouble(double i_value);
	CSDouble(const CSDouble& i_value);
	CSDouble(const CSBasicType& i_value);
	virtual ~CSDouble();

	void set(std::tr1::shared_ptr<CSBasicType> i_value, int i_pos);

	bool operator==(const CSDouble& i_object);
	bool operator!=(const CSDouble& i_object);

	friend std::ostream& operator<<(std::ostream& i_os, const CSDouble& i_object);

public:
	double m_value;
};

#endif /* CSDOUBLE_H_ */

#ifndef CS3DDPOINT_H_
#define CS3DDPOINT_H_

#include <CSBasicType.h>

class CS3DDPoint : public CSBasicType {
public:
	CS3DDPoint();
	CS3DDPoint(double i_x, double i_y, double i_z);
	CS3DDPoint(const CS3DDPoint& i_value);
	CS3DDPoint(const CSBasicType& i_value);
	virtual ~CS3DDPoint();

	bool operator==(const CS3DDPoint& i_object);
	bool operator!=(const CS3DDPoint& i_object);

	friend std::ostream& operator<<(std::ostream& i_os, const CS3DDPoint& i_object);

public:
	double m_value[3];
};

#endif /* CS3DDPOINT_H_ */

#ifndef CS3DIPOINT_H_
#define CS3DIPOINT_H_

#include <CSBasicType.h>

class CS3DIPoint : public CSBasicType {
public:
	CS3DIPoint();
	CS3DIPoint(const CS3DIPoint& i_value);
	CS3DIPoint(const CSBasicType& i_value);
	virtual ~CS3DIPoint();

	bool operator==(const CS3DIPoint& i_object);
	bool operator!=(const CS3DIPoint& i_object);

	friend std::ostream& operator<<(std::ostream& i_os, const CS3DIPoint& i_object);

public:
	int m_value[3];
};

#endif /* CS3DIPOINT_H_ */

#ifndef CS2DDPOINT_H_
#define CS2DDPOINT_H_

#include <CSBasicType.h>

class CS2DDPoint : public CSBasicType {
public:
	CS2DDPoint();
	CS2DDPoint(const CS2DDPoint& i_value);
	CS2DDPoint(const CSBasicType& i_value);
	CS2DDPoint(double, double);
	virtual ~CS2DDPoint();

	bool operator==(const CS2DDPoint& i_object);
	bool operator!=(const CS2DDPoint& i_object);

	friend std::ostream& operator<<(std::ostream& i_os, const CS2DDPoint& i_object);

public:
	double m_value[2];
};

#endif /* CS2DDPOINT_H_ */

#ifndef CS2DIPOINT_H_
#define CS2DIPOINT_H_

#include <CSBasicType.h>

class CS2DIPoint : public CSBasicType {
public:
	CS2DIPoint();
	CS2DIPoint(const CS2DIPoint& i_value);
	CS2DIPoint(const CSBasicType& i_value);
	virtual ~CS2DIPoint();

	bool operator==(const CS2DIPoint& i_object);
	bool operator!=(const CS2DIPoint& i_object);

	friend std::ostream& operator<<(std::ostream& i_os, const CS2DIPoint& i_object);

public:
	int m_value[2];
};

#endif /* CS2DIPOINT_H_ */

#ifndef CSTIMESTAMP_H_
#define CSTIMESTAMP_H_

#include <CSBasicType.h>

class CSTimeStamp : public CSBasicType {
public:
	CSTimeStamp();
	CSTimeStamp(int i_id, int i_tYear, int i_tMonth, int i_tDay, int i_tHour, int i_tMin, int i_tSec, int i_tMs);
	CSTimeStamp(int i_id, double i_second, double i_msecond);
	CSTimeStamp(const CSTimeStamp& i_value);
	CSTimeStamp(const CSBasicType& i_value);
	virtual ~CSTimeStamp();



	void computeTime();
	double duration(CSTimeStamp* i_time);
	double duration(CSTimeStamp i_time);
	double duration(std::tr1::shared_ptr<CSTimeStamp> i_time);

	bool operator==(const CSTimeStamp& i_object);
	bool operator!=(const CSTimeStamp& i_object);

	bool operator<(const CSTimeStamp& i_object);
	bool operator<=(const CSTimeStamp& i_object);
	bool operator>(const CSTimeStamp& i_object);
	bool operator>=(const CSTimeStamp& i_object);

	double operator+(const CSTimeStamp& i_object);
	double operator-(const CSTimeStamp& i_object);
	
	friend std::ostream& operator<<(std::ostream& i_os, const CSTimeStamp& i_object);

public:

	int m_id;

	int m_tYear;
	int m_tMonth;
	int m_tDay;
	int m_tHour;
	int m_tMin;
	int m_tSec;
	int m_tMs;

	double m_second;
	double m_msecond;
};

#endif /* CSTIMESTAMP_H_ */

#ifndef CSINTERVAL_H_
#define CSINTERVAL_H_

#include <CSBasicType.h>

class CSInterval : public CSBasicType {
public:

	CSInterval();
	CSInterval(const CSInterval& i_value);
	CSInterval(const CSBasicType& i_value);
	CSInterval(std::tr1::shared_ptr<CSTimeStamp> i_begin, std::tr1::shared_ptr<CSTimeStamp> i_end);
	virtual ~CSInterval();

	double duration();

	bool operator==(const CSInterval& i_object);
	bool operator!=(const CSInterval& i_object);


	friend std::ostream& operator<<(std::ostream& i_os, const CSInterval& i_object);

public:
	std::tr1::shared_ptr<CSTimeStamp> m_start;
	std::tr1::shared_ptr<CSTimeStamp> m_end;
};

#endif /* CSINTERVAL_H_ */

#ifndef CS3DDPOINTS_H_
#define CS3DDPOINTS_H_

#include <CSBasicType.h>

class CS3DDPoints : public CSBasicType {
public:
	CS3DDPoints();
	CS3DDPoints(const CS3DDPoints& i_value);
	CS3DDPoints(const CSBasicType& i_value);
	virtual ~CS3DDPoints();

	void addPoint(CS3DDPoint i_3dpoint);

	bool operator==(const CS3DDPoints& i_object);
	bool operator!=(const CS3DDPoints& i_object);

	friend std::ostream& operator<<(std::ostream& i_os, const CS3DDPoints& i_object);

public:
	std::vector<CS3DDPoint> m_value;
};

#endif /* CS3DDPOINTS_H_ */

#ifndef CSACCESSORAGENT_H_
#define CSACCESSORAGENT_H_

#include <CSBasicType.h>

class CSAccessorAgent : public CSBasicType {
public:
	CSAccessorAgent();
	CSAccessorAgent(const CSAccessorAgent& i_value);
	CSAccessorAgent(const CSBasicType& i_value);
	CSAccessorAgent(std::string i_agent, std::string i_attribute);
	virtual ~CSAccessorAgent();

	bool operator==(const CSAccessorAgent& i_object);
	bool operator!=(const CSAccessorAgent& i_object);

	friend std::ostream& operator<<(std::ostream& i_os, const CSAccessorAgent& i_object);

public:
	std::string m_agent;
	std::string m_attribute;
};



#endif /* CSACCESSORAGENT_H_ */






std::string toString(CSBasicType* i_object);
bool equal(std::tr1::shared_ptr<CSBasicType> i_left_object, std::tr1::shared_ptr<CSBasicType> i_right_object);
bool diff(std::tr1::shared_ptr<CSBasicType> i_left_object, std::tr1::shared_ptr<CSBasicType> i_right_object);


