// Alberto Avanzi - 10/03/2004

#ifndef _MATHFUNCTIONS_H_
#define _MATHFUNCTIONS_H_

namespace vsip 
{
  namespace math
  {
    template<typename T>
    const T square(const T& arg)
    {
      return arg*arg;
    }; 

    /* A normal distribution is
     *                                square(x-m)
     *                  1          - --------------
     * y(x,m,s) = ------------ * e    2*square(s)
     *            sqrt(2*pi*s)
     *
     * This functions returns only 
     *
     *                square(x-m)
     *              - -----------
     * y(x,m,s) = e   2*square(s)
     *
     * because we want that y(m,m,s) = 1. Remember that y(m+/-s,m,s) = 0.606
     */
    const double gaussian(const double& i_mean, const double& i_sigma, const double& i_x);
    
    const int roundInt(const double i_value);
    
    double ceilOrFloor(double value);

	bool areEqual(double value1, double value2);

	double max(double value1, double value2);
	int    max(int value1, int value2);

	double min(double value1, double value2);
	int    min(int value1, int value2);

  }
} // end namespace vsip

#endif // _MATHFUNCTIONS_H_
